
#include "ui/factory.h"
#include "ui/cntroler.h"
#include "ui/applic.h"
#include "ui/composit.h"
#include "ui/bmpushbn.h"
#include "ui/bmtglbtn.h"
#include "ui/cntroler.h"
#include "ui/pushbtn.h"
#include "ui/combobox.h"
#include "ui/toglbtn.h"
#include "ui/xrtglbtn.h"
#include "ui/stred.h"
#include "ui/dialog.h"
#include "ui/font.h"
#include "ui/label.h"
#include "ui/strview.h"
#include "ui/textedit.h"
#include "ui/scrolbar.h"
#include "ui/menu.h"
#include "ui/orbtngrp.h"
#include "ui/xrbtngrp.h"

#include "base/tree.h"


UI_VObjFactory::UI_VObjFactory ()
{
}


UI_VisualObject* UI_VObjFactory::_DoCreate (UI_ViewType type,
					    UI_VisualObject* parent,
					    const UI_Rectangle& shape,
					    UI_ViewID id,
					    const char* title,
					    UI_ViewDescriptor* enclosed)
{
    
    // First check if the parent already has a child with the given id
    CL_IntegerTree* viewTree = YACLApp()->Controller().ViewTree();
    CL_IntegerTreeNode* parentNode = viewTree->Node (parent->ViewID());
    CL_IntegerTreeNode* childNode = viewTree->Node (id);
    if (childNode && childNode->Parent() == parentNode) {
        CL_Error::Warning ("VObjFactory::DoCreate: Duplicate child id "
			   "%ld for composite",
                           id);
        return NULL;
    }

    // Now create the necessary visual object
    UI_VisualObject* p;
    switch (type) {

    case View_OrButtonGroup:
        p = new UI_OrButtonGroup (parent, shape, id, enclosed);
        p->Title () = title;
        break;

    case View_ExOrButtonGroup:
        p = new UI_ExOrButtonGroup (parent, shape, id, enclosed);
        p->Title () = title;
        break;

    case View_PushButton:
        p = new UI_PushButton (parent, shape, id);    
        p->Title()= title;
        break;

    case View_ToggleButton:
        p = new UI_ToggleButton (parent, shape, id);  
        p->Title() = title;
        break; 

    case View_ExOrToggleButton:
        p = new UI_ExOrToggleButton (parent, shape, id);  
        p->Title() = title;
        break; 

      
    case View_Label:
        p = new UI_Label (parent, shape, id);
        ((UI_Label*) p)->Title () = title;
        break;

    case View_StringEditor:
        p = new UI_StringEditor (parent, shape, id);
        ((CL_String&) (((UI_SimpleVObject*) p)->Model ())) = title;
        break;


    case View_TextView:
        p = new UI_TextEditor (parent, shape, id);
        ((CL_String&) (((UI_SimpleVObject*) p)->Model ())) = title;
        break;

    case View_StringViewSingle:
        p = new UI_StringViewSingleSel (parent, shape, id);
        break;
    
    case View_HScrollBar:
        p = new UI_HScrollBar (parent, shape, id);
        break;

    
    case View_VScrollBar:
        p = new UI_VScrollBar (parent, shape, id);
        break;


    case View_StringViewMulti:
        p = new UI_StringViewMultiSel (parent, shape, id);
        break;

    case View_ComboBox:
        p = new UI_ComboBox (parent, shape, id, shape.Height());
        break;

//     case View_Composite:
//         p = new UI_CompositeVObject (parent, enclosed, TRUE, shape,
//                                      id);
//         break;

    case View_BitmapPushButton:
        p = new UI_BitmapPushButton (parent, shape, id);
        break;

    case View_BitmapToggleButton:
        p = new UI_BitmapToggleButton (parent, shape, id);
        break;


    default:
        p = NULL;
        break;
    } // end of switch

//     if (p && type != View_Composite ) {
//         UI_ViewID id = p->ViewID();
//         if (id > 0 && id <= 32767) {
//             ((CL_IntPtrMap*) _model)->Add (id, &(p->Model()));
//             if (type != View_OrButtonGroup &&
//                 type != View_ExOrButtonGroup)
//                 ((UI_SimpleVObject*) p)->TabStop (tab_stop);
//         }
//     }
    return p;
}



